# Generated by roxygen2: do not edit by hand

S3method("[",sfc_nxn)
S3method("[",sfc_rules)
S3method("[",sfc_sequence)
S3method("[<-",sfc_sequence)
S3method("^",sfc_sequence)
S3method(c,sfc_sequence)
S3method(length,sfc_sequence)
S3method(makeContext,grob_sfc_sequence)
S3method(plot,sfc_base)
S3method(plot,sfc_nxn)
S3method(plot,sfc_sequence)
S3method(rev,sfc_sequence)
export(BASE_B)
export(BASE_C)
export(BASE_D)
export(BASE_I)
export(BASE_J)
export(BASE_L)
export(BASE_LIST)
export(BASE_P)
export(BASE_Q)
export(BASE_R)
export(BASE_U)
export(H0)
export(H1)
export(H2)
export(SFC_RULES_2x2)
export(SFC_RULES_3x3_COMBINED)
export(SFC_RULES_3x3_MEANDER)
export(SFC_RULES_3x3_PEANO)
export(SFC_RULES_4x4_MEANDER_1)
export(SFC_RULES_4x4_MEANDER_2)
export(add_base_structure)
export(all_2x2_shapes)
export(all_3x3_meander_shapes)
export(all_3x3_peano_shapes)
export(all_traverse_paths)
export(beta_omega_curve)
export(draw_multiple_curves)
export(draw_rules_2x2)
export(draw_rules_3x3_combined)
export(draw_rules_3x3_meander)
export(draw_rules_3x3_peano)
export(draw_rules_4x4_meander)
export(expand_h)
export(get_one_traverse_path)
export(h_curve)
export(hilbert_3d)
export(hilbert_curve)
export(meander_curve)
export(moore_curve)
export(peano_curve)
export(plot_segments)
export(plot_traverse_paths)
export(sfc_2x2)
export(sfc_3x3_combined)
export(sfc_3x3_meander)
export(sfc_3x3_peano)
export(sfc_4x4_meander)
export(sfc_base)
export(sfc_generator)
export(sfc_h)
export(sfc_rules)
export(sfc_seed)
export(sfc_sequence)
export(sfc_unit)
export(test_sfc_index)
export(unit_orientation)
exportMethods(change_level1_unit_orientation)
exportMethods(level1_unit_orientation)
exportMethods(sfc_apply)
exportMethods(sfc_dflip)
exportMethods(sfc_expand)
exportMethods(sfc_expand_by_rules)
exportMethods(sfc_flip_unit)
exportMethods(sfc_grob)
exportMethods(sfc_hflip)
exportMethods(sfc_index)
exportMethods(sfc_is_compatible)
exportMethods(sfc_level)
exportMethods(sfc_mode)
exportMethods(sfc_next_point)
exportMethods(sfc_previous_point)
exportMethods(sfc_reduce)
exportMethods(sfc_reverse)
exportMethods(sfc_rotate)
exportMethods(sfc_segments)
exportMethods(sfc_shape)
exportMethods(sfc_universe)
exportMethods(sfc_validate)
exportMethods(sfc_vflip)
exportMethods(show)
import(Rcpp)
import(colorRamp2)
import(graphics)
import(grid)
import(methods)
importFrom(stats,runif)
useDynLib(sfcurve, .registration = TRUE)
