# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

hilbert_curve_3d_cpp <- function(level) {
    .Call(`_sfcurve_hilbert_curve_3d_cpp`, level)
}

expand_by_rules_cpp <- function(rules, letters, code) {
    .Call(`_sfcurve_expand_by_rules_cpp`, rules, letters, code)
}

expand_by_rules_2_cpp <- function(rules, flip, letters, code, l_flip) {
    .Call(`_sfcurve_expand_by_rules_2_cpp`, rules, flip, letters, code, l_flip)
}

sfc_segments_cpp <- function(seq, rot, bases, start) {
    .Call(`_sfcurve_sfc_segments_cpp`, seq, rot, bases, start)
}

hilbert_curve_cpp <- function(level, type = 1L) {
    .Call(`_sfcurve_hilbert_curve_cpp`, level, type)
}

peano_curve_cpp <- function(level, type = 0L) {
    .Call(`_sfcurve_peano_curve_cpp`, level, type)
}

int_to_binary <- function(x, len) {
    .Call(`_sfcurve_int_to_binary`, x, len)
}

