---
title: "The sfcurve package"
author: "Zuguang Gu (guzuguang@suat-sz.edu.cn)"
date: '`r Sys.Date()`'
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{The sfcurve package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Please go to the package's website: https://jokergoo.github.io/sfcurve/.
